# -*- coding: utf-8 *-*
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Use SyncMenu lib to integrate U1 with the Systray Sync Icon."""

import gettext
import sys
import webbrowser

glib = None
try:
    if 'gobject' in sys.modules and sys.modules['gobject'] is not None:
        import glib as GLib
        glib = GLib
    else:
        from gi.repository import GLib
        glib = GLib
except ImportError:
    pass
try:
    from gi.repository import (
        Dbusmenu,
        SyncMenu,
    )
    use_syncmenu = True
except:
    use_syncmenu = False
from twisted.internet import reactor

from ubuntuone.clientdefs import GETTEXT_PACKAGE


Q_ = lambda string: gettext.dgettext(GETTEXT_PACKAGE, string)

OPEN_U1 = Q_("Open Ubuntu One")
GO_TO_WEB = Q_("Go to the Ubuntu One Website")
TRANSFERS = Q_("Current and Recent Transfers")
MORE_STORAGE = Q_("Get More Space")
GET_HELP = Q_("Get Help on the Web")

UBUNTUONE_LINK = u'https://one.ubuntu.com/'
DASHBOARD = UBUNTUONE_LINK + u'dashboard/'
HELP_LINK = UBUNTUONE_LINK + u'support/'
GET_STORAGE_LINK = UBUNTUONE_LINK + u'services/#storage_panel'


class UbuntuOneSyncMenuLinux(object):
    """Integrate U1 with the Ubuntu Sync Menu."""

    def __init__(self, main):
        """Initialize menu."""
        self._main = main
        self.status_listener = main.status_listener
        self._paused = False
        self.root_menu = Dbusmenu.Menuitem()

        self.open_u1 = Dbusmenu.Menuitem()
        self.open_u1.property_set(Dbusmenu.MENUITEM_PROP_LABEL, OPEN_U1)

        self.go_to_web = Dbusmenu.Menuitem()
        self.go_to_web.property_set(Dbusmenu.MENUITEM_PROP_LABEL,
            GO_TO_WEB)

        self.transfers = TransfersMenu(self.status_listener)
        self.transfers.property_set(Dbusmenu.MENUITEM_PROP_LABEL,
            TRANSFERS)

        self.more_storage = Dbusmenu.Menuitem()
        self.more_storage.property_set(Dbusmenu.MENUITEM_PROP_LABEL,
            MORE_STORAGE)

        self.get_help = Dbusmenu.Menuitem()
        self.get_help.property_set(Dbusmenu.MENUITEM_PROP_LABEL,
            GET_HELP)

        # Connect signals
        self.open_u1.connect(Dbusmenu.MENUITEM_SIGNAL_ITEM_ACTIVATED,
            self.open_control_panel)
        self.go_to_web.connect(Dbusmenu.MENUITEM_SIGNAL_ITEM_ACTIVATED,
            self.open_go_to_web)
        self.get_help.connect(Dbusmenu.MENUITEM_SIGNAL_ITEM_ACTIVATED,
            self.open_web_help)
        self.more_storage.connect(Dbusmenu.MENUITEM_SIGNAL_ITEM_ACTIVATED,
            self.open_get_more_storage)

        # Add items
        self.root_menu.child_append(self.open_u1)
        self.root_menu.child_append(self.go_to_web)
        self.root_menu.child_append(self.transfers)
        self.root_menu.child_append(self.more_storage)
        self.root_menu.child_append(self.get_help)

        self.server = Dbusmenu.Server()
        self.server.set_root(self.root_menu)
        self.app = SyncMenu.App.new("ubuntuone-installer.desktop")
        self.app.set_menu(self.server)
        self.app.connect("notify::paused", self.change_sync_status)

    def start_timer(self):
        """Start the transfers timer."""
        self.transfers.start_timer()

    def change_sync_status(self, *args):
        """Triggered when the sync status is changed fromm the menu."""
        if self._paused:
            self._main.external.connect()
            self._paused = False
        else:
            self._main.external.disconnect()
            self._paused = True

    def open_control_panel(self, *args):
        """Open the Ubuntu One Control Panel."""
        glib.spawn_command_line_async('ubuntuone-installer')

    def open_go_to_web(self, *args):
        """Open the Ubunto One Help Page"""
        webbrowser.open(DASHBOARD)

    def open_web_help(self, *args):
        """Open the Ubunto One Help Page"""
        webbrowser.open(HELP_LINK)

    def open_get_more_storage(self, *args):
        """Open the Ubunto One Help Page"""
        webbrowser.open(GET_STORAGE_LINK)


class TransfersMenu(Dbusmenu.Menuitem):
    """Menu that handles the recent and current transfers."""

    def __init__(self, listener):
        super(TransfersMenu, self).__init__()
        self.listener = listener
        self.uploading = {}
        self.previous_transfers = []
        self._transfers_items = {}
        self._uploading_items = {}

    def start_timer(self):
        """Trigger an update in one second."""
        self.update_progress()
        reactor.callLater(3, self.start_timer)

    def update_progress(self):
        """Update the list of recent transfers and current transfers."""
        current_transfers = self.listener.status_frontend.recent_transfers()
        uploading_data = {}
        for filename, size, written in \
            self.listener.status_frontend.files_uploading():
            uploading_data[filename] = (size, written)

        temp_transfers = {}
        if current_transfers != self.previous_transfers:
            for item_transfer in self._transfers_items:
                self.child_delete(self._transfers_items[item_transfer])
            for item in current_transfers:
                recent_file = Dbusmenu.Menuitem()
                recent_file.property_set(Dbusmenu.MENUITEM_PROP_LABEL,
                    item)
                self.child_add_position(recent_file, 0)
                temp_transfers[item] = recent_file
            self._transfers_items = temp_transfers

        items_added = 0
        remove = []
        for item in self._uploading_items:
            if item in uploading_data:
                size, written = uploading_data[item]
                percentage = written * 100 / size
                upload_item = self._uploading_items[item]
                upload_item.property_set_int(
                    SyncMenu.PROGRESS_MENUITEM_PROP_PERCENT_DONE,
                    percentage)
                items_added += 1
            else:
                self.child_delete(self._uploading_items[item])
                remove.append(item)
        for item in remove:
            self._uploading_items.pop(item)
        if items_added < 5:
            for item in uploading_data:
                if item not in self._uploading_items and items_added < 5:
                    size, written = uploading_data[item]
                    percentage = written * 100 / size
                    uploading_file = Dbusmenu.Menuitem()
                    uploading_file.property_set(Dbusmenu.MENUITEM_PROP_LABEL,
                        item)
                    uploading_file.property_set(Dbusmenu.MENUITEM_PROP_TYPE,
                        SyncMenu.PROGRESS_MENUITEM_TYPE)
                    uploading_file.property_set_int(
                        SyncMenu.PROGRESS_MENUITEM_PROP_PERCENT_DONE,
                        percentage)
                    self.child_append(uploading_file)
                    self._uploading_items[item] = uploading_file
                    items_added += 1


class DummySyncMenu(object):

    def start_timer(self):
        """Do nothing."""


UbuntuOneSyncMenu = UbuntuOneSyncMenuLinux if use_syncmenu else DummySyncMenu
